@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2019, Jice and the libtcod contributors.
+ * Copyright © 2008-2020, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,6 +48,8 @@
 #include "fov_types.h"
 #include "mersenne_types.h"
 #include "sys.h"
+#include "tileset.h"
+#include "context.h"
 
 /* tcodlib internal stuff */
 #ifdef __cplusplus
@@ -58,6 +60,7 @@ extern "C" {
 struct SDL_Surface;
 struct SDL_Window;
 struct SDL_Renderer;
+union SDL_Event;
 
 /* pseudorandom number generator toolkit */
 typedef struct TCOD_Random {
@@ -92,7 +95,6 @@ typedef struct {
 	int *ascii_to_tcod;
 	/* whether each character in the font is a colored tile */
 	bool *colored;
-#ifdef TCOD_CONSOLE_SUPPORT
 	/* the root console */
 	struct TCOD_Console *root;
 	/* nb chars in the font */
@@ -107,27 +109,26 @@ typedef struct {
 	/* actual resolution */
 	int actual_fullscreen_width;
 	int actual_fullscreen_height;
-#endif
-#ifndef TCOD_BARE
-	/* renderer to use */
-	TCOD_renderer_t renderer;
 	/* user post-processing callback */
 	SDL_renderer_t sdl_cbk;
-#endif
 	/* fading data */
 	TCOD_color_t fading_color;
 	uint8_t fade;
-#ifdef TCOD_CONSOLE_SUPPORT
 	TCOD_key_t key_state;
-#endif
-#ifndef TCOD_BARE
 	/* application window was closed */
 	bool is_window_closed;
 	/* application has mouse focus */
 	bool app_has_mouse_focus;
 	/* application is active (not iconified) */
 	bool app_is_active;
-#endif
+  /**
+    Active tileset for libtcod.
+
+    Use TCOD_set_default_tileset when you want to change this value, since it
+    needs to trigger some important side-effects.
+   */
+  struct TCOD_Tileset* tileset;
+  struct TCOD_Context* engine;
 } TCOD_internal_context_t;
 
 extern TCOD_internal_context_t TCOD_ctx;
@@ -161,27 +162,6 @@ extern TCOD_internal_context_t TCOD_ctx;
 #define TCOD_LOG(x) printf x
 #endif
 
-#if !defined(TCOD_BARE) && !defined(NO_OPENGL)
-/* opengl utilities */
-void TCOD_opengl_init_attributes(void);
-bool TCOD_opengl_init_state(int conw, int conh, struct SDL_Surface* font_tex);
-void TCOD_opengl_uninit_state(void);
-bool TCOD_opengl_init_shaders(void);
-bool TCOD_opengl_render(int oldFade, bool *ascii_updated, struct TCOD_Console *console, struct TCOD_Console *cache);
-void TCOD_opengl_swap(void);
-struct SDL_Surface* TCOD_opengl_get_screen(void);
-#endif
-
-#ifdef TCOD_IMAGE_SUPPORT
-/* image internal stuff */
-bool TCOD_image_mipmap_copy_internal(const TCOD_Image* srcImage,
-                                     TCOD_Image* dstImage);
-TCOD_color_t *TCOD_image_get_colors(TCOD_Image* image);
-void TCOD_image_invalidate_mipmaps(TCOD_Image* image);
-void TCOD_image_get_key_data(const TCOD_Image* image,
-                             bool *has_key_color, TCOD_color_t *key_color);
-#endif
-
 /* fov internal stuff */
 void TCOD_map_compute_fov_circular_raycasting(TCOD_map_t map, int player_x, int player_y, int max_radius, bool light_walls);
 void TCOD_map_compute_fov_diamond_raycasting(TCOD_map_t map, int player_x, int player_y, int max_radius, bool light_walls);
@@ -195,16 +175,9 @@ void TCOD_fatal(const char *fmt, ...);
 void TCOD_fatal_nopar(const char *msg);
 
 /* switch fullscreen mode */
-void TCOD_sys_set_fullscreen(bool fullscreen);
-void TCOD_sys_set_clear_screen(void);
-void TCOD_sys_set_scale_factor(float value);
-void TCOD_sys_convert_console_to_screen_coords(int cx, int cy, int *sx, int *sy);
-void TCOD_sys_convert_screen_to_console_coords(int sx, int sy, int *cx, int *cy);
-void TCOD_sys_flush(bool render);
 TCOD_key_t TCOD_sys_check_for_keypress(int flags);
 TCOD_key_t TCOD_sys_wait_for_keypress(bool flush);
 bool TCOD_sys_is_key_pressed(TCOD_keycode_t key);
-void TCOD_sys_set_window_title(const char *title);
 void TCOD_sys_pixel_to_tile(double* x, double* y);
 
 int TCOD_console_print_internal(
@@ -219,7 +192,6 @@ int TCOD_console_print_internal_utf(TCOD_console_t con,int x,int y, int rw, int
 	TCOD_alignment_t align, wchar_t *msg, bool can_split, bool count_only);
 #endif
 
-#ifdef TCOD_IMAGE_SUPPORT
 /* image manipulation */
 TCODLIB_API struct SDL_Surface* TCOD_sys_load_image(const char *filename);
 void TCOD_sys_get_image_size(const struct SDL_Surface *image, int *w,int *h);
@@ -227,12 +199,10 @@ TCOD_color_t TCOD_sys_get_image_pixel(const struct SDL_Surface *image,
                                       int x, int y);
 int TCOD_sys_get_image_alpha(const struct SDL_Surface* image, int x, int y);
 bool TCOD_sys_check_magic_number(const char *filename, size_t size, uint8_t *data);
-#endif
 
 /* TCOD_list nonpublic methods */
 void TCOD_list_set_size(TCOD_list_t l, int size);
 
-#ifndef TCOD_BARE
 /*
 	SDL12/SDL2 abstraction layer
 */
@@ -243,7 +213,7 @@ typedef struct TCOD_SDL_driver_t {
 	/* get a fullscreen mode suitable for the console */
 	void (*get_closest_mode)(int *w, int *h);
 	/* render the console on a surface/texture */
-	void (*render)(struct TCOD_SDL_driver_t *sdl, void *vbitmap, struct TCOD_Console *console);
+	void (*render)(const struct TCOD_SDL_driver_t *sdl, void *vbitmap, struct TCOD_Console *console);
 	/* create a new surface */
 	struct SDL_Surface *(*create_surface) (int width, int height, bool with_alpha);
 	/* create the game window */
@@ -273,16 +243,11 @@ typedef struct TCOD_SDL_driver_t {
 	struct TCOD_Console *(*get_root_console_cache)(void);
 } TCOD_SDL_driver_t;
 
-/* defined in TCOD_sys_sdl12_c.c and TCOD_sys_sdl2_c.c */
-TCOD_SDL_driver_t *SDL_implementation_factory(void);
-
-void find_resolution(void);
-void TCOD_sys_init_screen_offset(void);
 typedef struct {
 	float force_recalc;
 	float last_scale_xc, last_scale_yc;
 	float last_scale_factor;
-	float last_fullscreen;
+	bool last_fullscreen;
 
 	float min_scale_factor;
 
@@ -299,10 +264,6 @@ extern scale_data_t scale_data;
 
 extern float scale_factor;
 extern struct SDL_Surface* charmap;
-extern struct SDL_Window* window;
-extern struct SDL_Renderer* renderer;
-extern char *last_clipboard_text;
-#endif
 
 /* SDL & OpenGL */
 extern int oldFade;
@@ -520,100 +481,31 @@ extern int oldFade;
 #define TCOD_CELADON 172,255,175
 #define TCOD_PEACH 255,159,127
 
-#ifdef __cplusplus
-} // extern "C"
-#endif
-
-#ifdef __cplusplus
 // TCODConsole non public methods
-bool TCOD_console_init(TCOD_Console* con);
-bool TCOD_console_init(TCOD_Console* con, const std::string& title,
-                       bool fullscreen);
 int TCOD_console_stringLength(const unsigned char* s);
 unsigned char* TCOD_console_forward(unsigned char* s,int l);
 // TCODSystem non public methods
-bool TCOD_sys_init(struct TCOD_Console *console, bool fullscreen);
-void TCOD_sys_set_custom_font(const char *font_name,int nb_ch, int nb_cv,int flags);
+void sync_time_(void);
 void TCOD_sys_map_ascii_to_font(int asciiCode, int fontCharX, int fontCharY);
 void TCOD_sys_decode_font_(void);
-struct SDL_Surface* TCOD_sys_create_bitmap_for_console(TCOD_Console* console);
 void TCOD_sys_save_bitmap(struct SDL_Surface* bitmap, const char *filename);
 struct SDL_Surface* TCOD_sys_create_bitmap(int width, int height,
                                            TCOD_color_t *buf);
 void TCOD_sys_delete_bitmap(struct SDL_Surface* bitmap);
-void TCOD_sys_console_to_bitmap(
-    struct SDL_Surface* bitmap,
-    struct TCOD_Console *console,
-    struct TCOD_Console *cache);
-TCODLIB_CAPI struct SDL_Surface* TCOD_sys_get_surface(int width, int height,
-                                                      bool alpha);
 void TCOD_sys_save_fps(void);
 void TCOD_sys_restore_fps(void);
-void TCOD_sys_set_dirty(int dx, int dy, int dw, int dh);
-void TCOD_sys_set_dirty_character_code(int ch);
-int TCOD_get_tileid_for_charcode_(int charcode);
-namespace tcod {
-namespace console {
-/**
- *  Validate and return a console.
- */
-inline TCOD_Console* validate_(TCOD_Console* console)
-{
-  console = (console ? console : TCOD_ctx.root);
-  TCOD_ASSERT(console);
-  return console;
-}
+TCODLIB_CAPI TCOD_Error TCOD_sys_load_player_config(void);
 /**
  *  Validate and return a constant console.
  */
-inline const TCOD_Console* validate_(const TCOD_Console* console)
+static inline TCOD_Console* TCOD_console_validate_(const TCOD_Console* console)
 {
-  console = (console ? console : TCOD_ctx.root);
-  TCOD_ASSERT(console);
-  return console;
-}
-/**
- *  Return a console reference from a valid console pointer.
- */
-inline TCOD_Console& ensure_(TCOD_Console* console)
-{
-  console = validate_(console);
-  if (!console) {
-    throw std::logic_error("libtcod has not been initialized yet.");
-  }
-  return *console;
-}
-/**
- *  Return a console reference from a valid constant console pointer.
- */
-inline const TCOD_Console& ensure_(const TCOD_Console* console)
-{
-  console = validate_(console);
-  if (!console) {
-    throw std::logic_error("libtcod has not been initialized yet.");
-  }
-  return *console;
-}
-} // namespace console
-} // namespace tcod
-/**
- *  Validate and return a console.
- */
-inline TCOD_Console* TCOD_console_validate_(TCOD_Console* console)
-{
-  return tcod::console::validate_(console);
-}
-/**
- *  Validate and return a constant console.
- */
-inline const TCOD_Console* TCOD_console_validate_(const TCOD_Console* console)
-{
-  return tcod::console::validate_(console);
+  return (TCOD_Console*)(console ? console : TCOD_ctx.root);
 }
 /**
  *  Return true if the console is valid and the index is within it.
  */
-inline bool TCOD_console_is_index_valid_(const TCOD_Console* console,
+static inline bool TCOD_console_is_index_valid_(const TCOD_Console* console,
                                          int x, int y)
 {
   return console && 0 <= x && x < console->w && 0 <= y && y < console->h;
@@ -622,13 +514,7 @@ TCOD_event_t TCOD_sys_handle_mouse_event(
     const union SDL_Event* ev, TCOD_mouse_t* mouse);
 TCOD_event_t TCOD_sys_handle_key_event(
     const union SDL_Event* ev, TCOD_key_t* key);
-/**
- *  Private function, gets pointers of an old renderer.
- */
-struct SDL_Window* TCOD_sys_get_sdl_window_(void);
-/**
- *  Private function, gets pointers of an old renderer.
- */
-struct SDL_Renderer* TCOD_sys_get_sdl_renderer_(void);
-#endif // __cplusplus
+#ifdef __cplusplus
+} // extern "C"
+#endif
 #endif // TCODLIB_INT_H_
