@@ -1,6 +1,6 @@
 /* Python specific cdefs which are loaded directly into cffi. */
+#include "../libtcod/src/libtcod/libtcod.h"
 extern "Python" {
-
 bool _pycall_parser_new_struct(TCOD_parser_struct_t str, const char *name);
 bool _pycall_parser_new_flag(const char *name);
 bool _pycall_parser_new_property(
@@ -18,4 +18,5 @@ float _pycall_path_dest_only(int x1, int y1, int x2, int y2, void *user_data);
 
 void _pycall_sdl_hook(struct SDL_Surface*);
 
+int _pycall_event_watch(void* userdata, union SDL_Event* event);
 }
