@@ -1,9 +1,9 @@
 /* This header is the entry point for the cffi parser.
-   Anything included here will be accessible from tcod.libtcod.lib */
+   Anything included here will be accessible from tcod.loader.lib */
 #include "../libtcod/src/libtcod/libtcod.h"
 #include "../libtcod/src/libtcod/libtcod_int.h"
 #include "../libtcod/src/libtcod/wrappers.h"
-#include "../libtcod/src/libtcod/tileset/truetype.h"
+#include "../libtcod/src/libtcod/tileset_truetype.h"
 
 #include "noise.h"
 #include "path.h"
